<?php
/*
==========================================================
	This software package is intended for use with 
	ExpressionEngine.	ExpressionEngine is Copyright © 
	2002-2009 EllisLab, Inc. 
	http://ellislab.com/
==========================================================
	THIS IS COPYRIGHTED SOFTWARE, All RIGHTS RESERVED.
	Written by: Justin Crawford
	Copyright (c) 2009 Hop Studios
	http://www.hopstudios.com/software/
--------------------------------------------------------
	Please do not distribute this software without written
	consent from the author.
==========================================================
	Files:
	- pi.edit_this.php
----------------------------------------------------------
	Purpose: 
	- adds an edit_this link to rendered templates, if user has permissions to edit 
==========================================================
*/

$plugin_info = array(
  'pi_name' => 'Edit This',
  'pi_version' => '1.1.1',
  'pi_author' => 'Justin Crawford (Hop Studios)',
  'pi_author_url' => 'http://www.hopstudios.com/software',
  'pi_description' => 'Adds an "edit this" link to various content',
  'pi_usage' => Edit_this::usage()
);

class Edit_this
{

	var $icon_path = '';
	var $session_required = FALSE;
	var $debug = FALSE;
	
	// constructor
	function Edit_this() 
	{
		global $PREFS, $TMPL;
		$this->icon_path = $PREFS->ini('theme_folder_url') . 'edit_this_themes/images/';
		$this->debug = ($TMPL->fetch_param('debug') == 'true') ? TRUE : FALSE;
		$this->_debug("initializing");
	}

	// _get_link: create an href and wrap it around an image
	function _get_link($url = '', $alt = '', $icon = 'default') 
	{
		$this->icon_path .= (($icon == '') ? 'default' : $icon) . '_icon.gif';
		$this->_debug("creating a link with '$url'");
		return '<a href="' . $url . '" title="' . $alt . '" target="_blank"><img src="' . $this->icon_path . '" height="14" width="14" style="padding-left: 4px;" alt="' . $alt . '" align="top" border="0" /></a>';
	}

	// _session_ok: check whether a session ID is necessary, and get it if it exists
	function _session_ok()
	{
		global $PREFS, $SESS, $DB;

		// check if if we need an admin session id
		if ($PREFS->ini('admin_session_type') != 'c')
		{
			$this->session_required = TRUE;

			// if we haven't already got one, get one and cache it
			if (! isset($SESS->cache['edit_this']['session_id']))
			{
				$query = $DB->query("SELECT session_id FROM exp_sessions WHERE member_id = '" . $SESS->userdata['member_id'] . "' AND admin_sess = '1'");		
				if ($query->num_rows > 0)
				{
					$this->_debug("found an admin session id -- caching it");
					$SESS->cache['edit_this']['session_id'] = $query->row['session_id'];
				}
				else
				{
					// if admin sessions are required, but we didn't find an admin session
					$this->_debug("admin session check: FALSE");
					return FALSE;
				}
			}
		}
		else
		{
			$this->session_required = FALSE;
		}
		
		// either we found an admin session, or we don't need one
		$this->_debug("admin session check: TRUE");
		return TRUE;
	}

	function _debug($msg)
	{
		if ($this->debug === TRUE)
		{
			// do something with debug msgs; this is rudimentary, but works.
			echo "edit_this -- " . $msg . "<br />";
		}
	}

	// authz: if superadmin, $can_edit
	function _authz_superadmin()
	{
		global $SESS;
		if ($SESS->userdata['group_id'] == 1)
		{
			$this->_debug("superadmin check: TRUE");
			return TRUE;
		}
		$this->_debug("superadmin check: FALSE");
		return FALSE;
	}

	// entry: return a link or a url for an edit_this:entry tag
	function entry() 
	{
		global $TMPL, $PREFS, $SESS, $DB;
		$this->_debug("handling an entry");

		$can_edit = FALSE;

		// there are three required tag parameters, all numeric
		if ((is_numeric($entry_id = ($TMPL->fetch_param('entry_id')))) && is_numeric($weblog_id = ($TMPL->fetch_param('weblog_id'))) && is_numeric($author_id = ($TMPL->fetch_param('author_id'))))
		{

			$this->_debug("parameters OK");

			// verify that they either have an admin session, or don't need one
			if ($this->_session_ok() === TRUE)
			{
				if ($this->_authz_superadmin() === TRUE)
				{
					$can_edit = TRUE;
				}
				else 
				{
					// check the cache for authz for this weblog
					if (! isset($SESS->cache['edit_this']['weblog:' . $weblog_id]))
					{
	
						// authz: check that this person's group can edit this weblog
						$query = $DB->query("SELECT wt.weblog_id, wt.author_id FROM exp_weblog_member_groups wg, exp_weblog_titles wt WHERE wt.entry_id = " . $entry_id . " AND wg.weblog_id = wt.weblog_id AND wg.group_id = " . $SESS->userdata['group_id']);
	
						// cache the result of this authz check
						if ($query->num_rows > 0)
						{
							// they passed us a weblog/author id, but the DB is more definitive
							$weblog_id = $query->row['weblog_id'];
							$author_id = $query->row['author_id'];

							$SESS->cache['edit_this']['weblog:' . $weblog_id] = TRUE;
						}
						else
						{
							$SESS->cache['edit_this']['weblog:' . $weblog_id] = FALSE;
						}
						$this->_debug("caching result of weblog group authz for weblog $weblog_id");
					}
					
					// if we've cached affirmative authz for this weblog...
					if ($SESS->cache['edit_this']['weblog:' . $weblog_id] === TRUE)
					{
						$this->_debug("weblog group check: TRUE");
	
						// they must have these perms to edit 
						$can_edit = ($SESS->userdata['can_access_cp'] == 'y') && ($SESS->userdata['can_access_edit'] == 'y');	
						$this->_debug("control panel permission check: $can_edit");
	
						// if not the author, they must have extra permissions to edit others' entries
						if ($author_id != $SESS->userdata('member_id'))
						{
							$can_edit = $can_edit && ($SESS->userdata['can_edit_other_entries'] == 'y');	
							$this->_debug("non-author permission check: $can_edit");
						}
					}
				}
			}
			
			// if we got through all the above an still $can_edit, then let's do it!
			if ($can_edit)
			{
				// if admin sessions are required, get the session id
				$session_param = ($this->session_required == TRUE) ? '&amp;S=' . $SESS->cache['edit_this']['session_id'] : '';

				// generate the url
				$url = $PREFS->core_ini['cp_url'] . '?C=edit&amp;M=edit_entry&amp;entry_id=' . $entry_id . '&amp;weblog_id=' . $weblog_id . $session_param;

				// if invoked with two-tag, handle contents
				if ($TMPL->tagdata != '')
				{ 
					return str_replace('{edit_this_url}', $url, $TMPL->tagdata);  
				}
				else 
				{
					$alt = "Edit this item";
					return $this->_get_link($url, $alt, $TMPL->fetch_param('icon'));
				}
			}
		}
		return;
	}

	// template: return a url or a link for an edit_this:template tag
	function template()
	{
		global $TMPL, $PREFS, $SESS, $DB;
		$this->_debug("handling a template");

		$can_edit = FALSE;

		if (is_numeric($template_id = ($TMPL->fetch_param('template_id'))) && is_numeric($template_group = ($TMPL->fetch_param('template_group'))))
		{

			$this->_debug("parameters OK");

			// verify that they either have an admin session, or don't need one
			if ($this->_session_ok() === TRUE)
			{

				if ($this->_authz_superadmin())
				{
					$can_edit = TRUE;
				}
				else 
				{
					// check the cache for a template group authz
					if (! isset($SESS->cache['edit_this']['template_group:' . $template_group]))
					{
						// try to get authz on this template group
						$query = $DB->query("SELECT tt.group_id FROM exp_template_member_groups tg, exp_templates tt WHERE tt.template_id = " . $template_id . " AND tg.template_group_id = tt.group_id AND tg.group_id = " . $SESS->userdata['group_id']);

						if ($query->num_rows > 0)
						{
							// they passed us a template group id, but the DB is more definitive
							$template_group = $query->row['group_id'];

							$SESS->cache['edit_this']['template_group:' . $template_group] = TRUE;
						}
						else
						{
							$SESS->cache['edit_this']['template_group:' . $template_group] = FALSE;
						}
						$this->_debug("caching result of template group authz for template group $template_group");
					}

					// if template group authz succeeded
					if ($SESS->cache['edit_this']['template_group:' . $template_group] === TRUE)
					{
						$this->_debug("template group check: TRUE");

						// check other required template permissions
						$can_edit = ($SESS->userdata['can_access_cp'] == 'y')	&& ($SESS->userdata['can_admin_templates'] == 'y');	
						$this->_debug("control panel permission check: $can_edit");
					}
				}
			}
		
			if ($can_edit)
			{
				// if admin sessions are required, get the session id
				$session_param = ($this->session_required == TRUE) ? '&amp;S=' . $SESS->cache['edit_this']['session_id'] : '';

				$url = $PREFS->core_ini['cp_url'] . '?C=templates&amp;M=edit_template&amp;id=' . $template_id . $session_param;
				if ($TMPL->tagdata != '')
				{ 
					return str_replace('{edit_this_url}', $url, $TMPL->tagdata);  
				}
				else 
				{	
					$alt = "Edit this template";
					return $this->_get_link($url, $alt, $TMPL->fetch_param('icon'));
				}
			}
		}
		return;
	}


	// category: return a url or a link for an edit_this:category tag
	function category()
	{
		global $TMPL, $PREFS, $DB, $SESS;
		$this->_debug("handling a category");
		$can_edit = FALSE;

		if (is_numeric($category_id = ($TMPL->fetch_param('category_id'))) && is_numeric($category_group = ($TMPL->fetch_param('category_group'))))
		{
			$this->_debug("parameters OK");

			// verify that they either have an admin session, or don't need one
			if ($this->_session_ok() === TRUE)
			{

				if ($this->_authz_superadmin() === TRUE)
				{
					$can_edit = TRUE;
				}
				else 
				{
					// they must have these perms to edit 
					$can_edit = ($SESS->userdata['can_access_cp'] == 'y')	&& ($SESS->userdata['can_access_admin'] == 'y') && ($SESS->userdata['can_admin_weblogs'] == 'y');
					$this->_debug("control panel permission check: $can_edit");
					// Note: there is a perm 'can_edit_categories', but it doesn't apply in this instance.  It's for editing categories on a particular entry.
				}
			
				if ($can_edit)
				{
					// if admin sessions are required, get the session id
					$session_param = ($this->session_required == TRUE) ? '&amp;S=' . $SESS->cache['edit_this']['session_id'] : '';

					$url = $PREFS->core_ini['cp_url'] . '?C=admin&amp;M=blog_admin&amp;P=edit_category&amp;cat_id=' . $category_id . '&amp;group_id=' . $category_group . $session_param;
					if ($TMPL->tagdata != '')
					{ 
						return str_replace('{edit_this_url}', $url, $TMPL->tagdata);  
					}
					else 
					{	
						$alt = "Edit this category";
						return $this->_get_link($url, $alt, $TMPL->fetch_param('icon'));
					}
				}
			}
			return;
		}
	}	

	// image: return a url or a link for an edit_this:image tag
	function image() 
	{
		global $TMPL, $PREFS, $DB, $SESS;
		$this->_debug("handling an image");
		$can_edit = FALSE;

		if (is_numeric($entry_id = ($TMPL->fetch_param('entry_id'))) && is_numeric($gallery_id = ($TMPL->fetch_param('gallery_id'))))
		{
			$this->_debug("parameters OK");

			// verify that they either have an admin session, or don't need one
			if ($this->_session_ok() === TRUE)
			{

				if ($this->_authz_superadmin() === TRUE)
				{
					$can_edit = TRUE;
				}
				else 
				{
					// they must be authorized to admin the gallery module.  cached to eliminate redundant queries.
					if (! isset($SESS->cache['edit_this']['gallery']))
					{
						$query = $DB->query("SELECT * FROM exp_module_member_groups mg, exp_modules mm WHERE mg.module_id = mm.module_id AND mm.module_name = 'Gallery' AND mg.group_id = " . $SESS->userdata['group_id']);

						// unlike weblogs, template groups, there are no more granular permissions to cache.  they either can or cannot admin gallery module.
						$SESS->cache['edit_this']['gallery'] = $query->num_rows > 0 ? TRUE : FALSE;
						$this->_debug("caching result of gallery authz");
					}

					if ($SESS->cache['edit_this']['gallery'] === TRUE)
					{
						// they must have these perms to edit 
						$can_edit = ($SESS->userdata['can_access_cp'] == 'y') && ($SESS->userdata['can_access_modules'] == 'y');	
						$this->_debug("control panel permission check: $can_edit");
					}
				}
			
				if ($can_edit)
				{
					// if admin sessions are required, get the session id
					$session_param = ($this->session_required == TRUE) ? '&amp;S=' . $SESS->cache['edit_this']['session_id'] : '';

					$url = $PREFS->core_ini['cp_url'] . '?C=modules&amp;M=gallery&amp;P=entry_form&amp;gallery_id=' . $gallery_id . '&amp;entry_id=' . $entry_id . $session_param;
					if ($TMPL->tagdata != '')
					{ 
						return str_replace('{edit_this_url}', $url, $TMPL->tagdata);  
					}
					else 
					{	
						$alt = "Edit this image";
						return $this->_get_link($url, $alt, $TMPL->fetch_param('icon'));
					}
				}
			}
			return;
		}
	}
	
  // ----------------------------------------
  //  Plugin Usage
  // ----------------------------------------

  // This function describes how the plugin is used.
  //  Make sure and use output buffering

  function usage()
  {
  ob_start(); 
  ?>

This plugin will put small pencil icons in the live site, but only for users who are logged in, have an administrative session, and have permission to edit that particular resource.

All parameters are required, except "icon".

The simplest usage is simply this tag without any enclosed text:

Entries:
{exp:edit_this:entry entry_id="{entry_id}" weblog_id="{weblog_id}" author_id="{author_id}"} 

Templates:
{exp:edit_this:template template_id="42" template_group="2"} 

Categories:
{exp:edit_this:category category_id="{category_id}" category_group="{category_group}"}

Gallery Images:
{exp:edit_this:image entry_id="{entry_id}"}

If you wish to use a different icon image, several different colors are provided (in the /themes/edit_this_themes/images directory).  You can even put your own image there, as long as the filename ends in “_icon.gif”.  Call alternate icons like so:
{exp:edit_this:template template_id="42" template_group="2" icon="blue"}

Advanced Use:

You can use the same tag, but enclose some text within it.  The plugin will do the permission check, but leave the presentation of the link to you. This allows you to change the display to whatever you'd like, and you could have entire instructions or alternate content displayed by this plugin.

You can even point to a front-end editing template (stand alone editing form) using this option.

{exp:edit_this:entry entry_id="{entry_id}" weblog_id="{weblog_id}" author_id="{author_id}"} 
<a href="{edit_this_url}">I can edit this</a>
{/exp:edit_this:entry}

{exp:edit_this:template template_id="42" template_group="2"} 
<a href="{edit_this_url}">I can edit this</a>
{/exp:edit_this:template}

{exp:edit_this:category category_id="{category_id}" category_group="{category_group}"}
<a href="{edit_this_url}">I can edit this</a>
{/exp:edit_this:category}

{exp:edit_this:image entry_id="{entry_id}"}
<a href="{edit_this_url}">I can edit this</a>
{/exp:edit_this:image}

Image editing only applies to images in the gallery module.

	<?php
  $buffer = ob_get_contents();
	
  ob_end_clean(); 

  return $buffer;
  }
  // END

}
