<?php
/*
==========================================================
	This software package is intended for use with 
	ExpressionEngine.	ExpressionEngine is Copyright © 
	2002-2011 EllisLab, Inc. 
	http://ellislab.com/
==========================================================
	THIS IS COPYRIGHTED SOFTWARE, All RIGHTS RESERVED.
	Written by: Justin Crawford and Travis Smith
	Copyright (c) 2014 Hop Studios
	http://www.hopstudios.com/software/
--------------------------------------------------------
	Please do not distribute this software without written
	consent from the author.
==========================================================
*/

if ( ! defined('BASEPATH')) exit('No direct script access allowed');

require_once PATH_THIRD."edit_this/config.php";

$plugin_info = array(
  'pi_name'			=> 'Edit This',
  'pi_version'		=> EDIT_THIS_VERSION,
  'pi_author' 		=> 'Travis Smith (Hop Studios)',
  'pi_author_url' 	=> 'http://www.hopstudios.com/software/',
  'pi_description' 	=> 'Adds an "edit this" link to various content',
  'pi_usage' 		=> Edit_this::usage()
);

/**
 * Edit_this Class
 *
 * @package			ExpressionEngine
 * @category		Plugin
 * @author			Justin Crawford and Travis Smith
 * @copyright		Copyright (c) 2014, Hopstudios
 * @link			http://www.hopstudios.com/software
 */

class Edit_this
{

	var $icon_path = '';
	var $session_required = FALSE;
	var $debug = FALSE;
	
	// constructor
	function Edit_this() 
	{
		$this->EE =& get_instance();

		$this->icon_path = $this->EE->config->item('theme_folder_url') . 'third_party/edit_this/images/';
		$this->debug = ($this->EE->TMPL->fetch_param('debug') == 'true') ? TRUE : FALSE;
		$this->_debug("initializing");
	}

	// _get_link: create an href and wrap it around an image
	function _get_link($url = '', $alt = '', $icon = 'default') 
	{
		$this->icon_path .= (($icon == '') ? 'default' : $icon) . '_icon.gif';
		$this->_debug("creating a link with '$url'");
		return '<a href="' . $url . '" title="' . $alt . '" target="_blank"><img src="' . $this->icon_path . '" height="14" width="14" class="edit_this" style="padding-left: 4px;" alt="' . $alt . '" align="top" border="0" /></a>';
	}

	// _session_ok: check whether a session ID is necessary, and get it if it exists
	function _session_ok()
	{
		$this->EE =& get_instance();

		// check if if we need an admin session id
		if ($this->EE->config->item('admin_session_type') != 'c')
		{
			$this->session_required = TRUE;

			// if we haven't already got one, get one and cache it
			if (! isset($this->EE->session->cache['edit_this']['session_id']))
			{
				//Orig: "SELECT session_id FROM exp_sessions WHERE member_id = '" . $this->EE->session->userdata['member_id'] . "' AND admin_sess = '1'"
				$this->EE->db->select('session_id, fingerprint');
				$this->EE->db->from('exp_sessions');
				$this->EE->db->where('member_id', $this->EE->session->userdata('member_id'));
				$this->EE->db->where('admin_sess', '1');
				$query = $this->EE->db->get();

				if ($query->num_rows() > 0)
				{
					$this->_debug("found an admin session id -- caching it");
					// fingerprint was introduced in 2.5.4, otherwise use session_id
					if ( (version_compare(APP_VER, '2.5.4', '>=')) && ($this->EE->config->item('admin_session_type') == 'cs') )
					{
						$this->EE->session->cache['edit_this']['session_id'] = $query->row('fingerprint');
						$this->_debug("fingerprint: " . $query->row('fingerprint'));
					}
					else
					{
						$this->EE->session->cache['edit_this']['session_id'] = $query->row('session_id');
						$this->_debug("session_id: " . $query->row('session_id'));
					}
				}
				else
				{
					// if admin sessions are required, but we didn't find an admin session
					$this->_debug("admin session check: FALSE");
					return FALSE;
				}
			}
		}
		else
		{
			$this->session_required = FALSE;
		}
		
		// either we found an admin session, or we don't need one
		$this->_debug("admin session check: TRUE");
		return TRUE;
	}

	function _debug($msg)
	{
		if ($this->debug === TRUE)
		{
			// do something with debug msgs; this is rudimentary, but works.
			echo "edit_this -- " . $msg . "<br />";
		}
	}

	// authz: if superadmin, $can_edit
	function _authz_superadmin()
	{
		$this->EE =& get_instance();
		if ($this->EE->session->userdata('group_id') == 1)
		{
			$this->_debug("superadmin check: TRUE");
			return TRUE;
		}
		$this->_debug("superadmin check: FALSE");
		return FALSE;
	}

	// entry: return a link or a url for an edit_this:entry tag
	function entry() 
	{
		$this->EE =& get_instance();
		$this->_debug("handling an entry");

		$can_edit = FALSE;

		// there are three required tag parameters, all numeric
		if ((is_numeric($entry_id = ($this->EE->TMPL->fetch_param('entry_id')))) && is_numeric($channel_id = ($this->EE->TMPL->fetch_param('channel_id'))) && is_numeric($author_id = ($this->EE->TMPL->fetch_param('author_id'))))
		{

			$this->_debug("parameters OK");

			// verify that they either have an admin session, or don't need one
			if ($this->_session_ok() === TRUE)
			{
				if ($this->_authz_superadmin() === TRUE)
				{
					$can_edit = TRUE;
				}
				else 
				{
					// check the cache for authz for this channel
					if (! isset($this->EE->session->cache['edit_this']['channel:' . $channel_id]))
					{
	
						// authz: check that this person's group can edit this channel
						//orig: "SELECT wt.channel_id, wt.author_id FROM exp_weblog_member_groups wg, exp_weblog_titles wt WHERE wt.entry_id = " . $entry_id . " AND wg.channel_id = wt.channel_id AND wg.group_id = " . $this->EE->session->userdata['group_id']
						$this->EE->db->select('cg.channel_id');
						$this->EE->db->from('exp_channel_member_groups AS cg');
						$this->EE->db->where('cg.channel_id', $channel_id);
						$this->EE->db->where('cg.group_id', $this->EE->session->userdata('group_id'));
						$query = $this->EE->db->get();

						// cache the result of this authz check
						if ($query->num_rows() > 0)
						{
							if (is_array($query->result_array()))
							{
								foreach($query->result_array() as $row)
								{
									// they passed us a channel/author id, but the DB is more definitive
									$channel_id = $row['channel_id'];
		
									$this->EE->session->cache['edit_this']['channel:' . $channel_id] = TRUE;
								}
							}
						}
						else
						{
							$this->EE->session->cache['edit_this']['channel:' . $channel_id] = FALSE;
						}
						$this->_debug("caching result of channel group authz for channel $channel_id");
					}

					
					// if we've cached affirmative authz for this channel...
					if ($this->EE->session->cache['edit_this']['channel:' . $channel_id] === TRUE)
					{
						$this->_debug("channel group check: TRUE");
	
						// they must have these perms to edit 
						$can_edit = ($this->EE->session->userdata('can_access_cp') == 'y') && ($this->EE->session->userdata('can_access_edit') == 'y');	
						$this->_debug("control panel permission check: $can_edit");
	
						// if not the author, they must have extra permissions to edit others' entries
						if ($author_id != $this->EE->session->userdata('member_id'))
						{
							$can_edit = $can_edit && ($this->EE->session->userdata('can_edit_other_entries') == 'y');	
							$this->_debug("non-author permission check: $can_edit");
						}
					}
				}
			}
			
			// if we got through all the above an still $can_edit, then let's do it!
			if ($can_edit)
			{
				// if admin sessions are required, get the session id
				$session_param = 'S=' . (($this->session_required === TRUE) ? $this->EE->session->cache['edit_this']['session_id'] : '0') . '&amp;';

				// generate the url
				// http://localhost:8888/system/index.php?S=0&D=cp&C=content_publish&M=entry_form&channel_id=9&entry_id=3887
				$url = $this->EE->config->item('cp_url') . '?' . $session_param . 'D=cp&amp;C=content_publish&amp;M=entry_form&amp;channel_id=' . $channel_id . '&amp;entry_id=' . $entry_id ;
				// cp_url uses SELF which doesn't quite work on the front side, so there's an extra correction needed
				if (version_compare(APP_VER, '2.8', '>='))
				{
					ee()->load->helper('url_helper');
					$url = cp_url('content_publish/entry_form', array('channel_id' => $channel_id, 'entry_id' => $entry_id));
					$url = str_replace(SELF,$this->EE->config->item('cp_url'),$url);
				}

				// if invoked with two-tag, handle contents
				if ($this->EE->TMPL->tagdata != '')
				{ 
					return str_replace('{edit_this_url}', $url, $this->EE->TMPL->tagdata);  
				}
				else 
				{
					$alt = "Edit this item";
					return $this->_get_link($url, $alt, $this->EE->TMPL->fetch_param('icon'));
				}
			}
		}
		return;
	}

	// template: return a url or a link for an edit_this:template tag
	function template()
	{
		$this->EE =& get_instance();
		$this->_debug("handling a template");

		$can_edit = FALSE;

		if (is_numeric($template_id = ($this->EE->TMPL->fetch_param('template_id'))) && is_numeric($template_group = ($this->EE->TMPL->fetch_param('template_group'))))
		{

			$this->_debug("parameters OK");

			// verify that they either have an admin session, or don't need one
			if ($this->_session_ok() === TRUE)
			{

				if ($this->_authz_superadmin())
				{
					$can_edit = TRUE;
				}
				else 
				{
					// check the cache for a template group authz
					if (! isset($this->EE->session->cache['edit_this']['template_group:' . $template_group]))
					{
						// try to get authz on this template group
						//orig: "SELECT tt.group_id FROM exp_template_member_groups tg, exp_templates tt WHERE tt.template_id = " . $template_id . " AND tg.template_group_id = tt.group_id AND tg.group_id = " . $this->EE->session->userdata['group_id'];
						$this->EE->db->select('tt.group_id');
						$this->EE->db->from('exp_template_member_groups AS tg');
						$this->EE->db->from('exp_templates AS tt');
						$this->EE->db->where('tt.template_id', $template_id);
						$this->EE->db->where('tg.template_group_id = tt.group_id');
						$this->EE->db->where('tg.group_id', $this->EE->session->userdata('group_id'));
						$query = $this->EE->db->get();
						
						if ($query->num_rows() > 0)
						{
							if (is_array($query->result_array()))
							{
								foreach($query->result_array() as $row)
								{
									// they passed us a template group id, but the DB is more definitive
									$template_group = $row['group_id'];
		
									$this->EE->session->cache['edit_this']['template_group:' . $template_group] = TRUE;
								}
							}
						}
						else
						{
							$this->EE->session->cache['edit_this']['template_group:' . $template_group] = FALSE;
						}
						$this->_debug("caching result of template group authz for template group $template_group");
					}

					// if template group authz succeeded
					if ($this->EE->session->cache['edit_this']['template_group:' . $template_group] === TRUE)
					{
						$this->_debug("template group check: TRUE");

						// check other required template permissions
						$can_edit = ($this->EE->session->userdata('can_access_cp') == 'y')	&& ($this->EE->session->userdata('can_admin_templates') == 'y');	
						$this->_debug("control panel permission check: $can_edit");
					}
				}
			}
		
			if ($can_edit)
			{
				// if admin sessions are required, get the session id
				$session_param = 'S=' . (($this->session_required === TRUE) ? $this->EE->session->cache['edit_this']['session_id'] : '0') . '&amp;';

				// http://localhost:8888/system/index.php?S=0&D=cp&C=design&M=edit_template&id=7
				$url = $this->EE->config->item('cp_url') . '?' . $session_param . 'D=cp&amp;C=design&amp;M=edit_template&amp;id=' . $template_id ;
				if (version_compare(APP_VER, '2.8', '>='))
				{
					ee()->load->helper('url_helper');
					$url = cp_url('design/edit_template', array('id' => $template_id));
					$url = str_replace(SELF,$this->EE->config->item('cp_url'),$url);
				}

				if ($this->EE->TMPL->tagdata != '')
				{ 
					return str_replace('{edit_this_url}', $url, $this->EE->TMPL->tagdata);  
				}
				else 
				{	
					$alt = "Edit this template";
					return $this->_get_link($url, $alt, $this->EE->TMPL->fetch_param('icon'));
				}
			}
		}
		return;
	}

	// category: return a url or a link for an edit_this:category tag
	function category()
	{
		$this->EE =& get_instance();
		$this->_debug("handling a category");

		$can_edit = FALSE;

		if (is_numeric($category_id = ($this->EE->TMPL->fetch_param('category_id'))) && is_numeric($category_group = ($this->EE->TMPL->fetch_param('category_group'))))
		{
			$this->_debug("parameters OK");

			// verify that they either have an admin session, or don't need one
			if ($this->_session_ok() === TRUE)
			{

				if ($this->_authz_superadmin() === TRUE)
				{
					$can_edit = TRUE;
				}
				else 
				{
					// they must have these perms to edit 
					$can_edit = ($this->EE->session->userdata('can_access_cp') == 'y')	&& ($this->EE->session->userdata('can_access_admin') == 'y') && ($this->EE->session->userdata('can_access_content_prefs') == 'y');
					$this->_debug("control panel permission check: $can_edit");
					// Note: there is a perm 'can_edit_categories', but it doesn't apply in this instance.  It's for editing categories on a particular entry.
				}
			
				if ($can_edit)
				{
					// if admin sessions are required, get the session id
					$session_param = 'S=' . (($this->session_required === TRUE) ? $this->EE->session->cache['edit_this']['session_id'] : '0') . '&amp;';

					// http://localhost:8888/system/index.php?S=0&D=cp&C=admin_content&M=category_edit&cat_id=5&group_id=2
					$url = $this->EE->config->item('cp_url') . '?' . $session_param . 'D=cp&amp;C=admin_content&amp;M=category_edit&amp;cat_id=' . $category_id . '&amp;group_id=' . $category_group ;
					if (version_compare(APP_VER, '2.8', '>='))
					{
						ee()->load->helper('url_helper');
						$url = cp_url('admin_content/category_edit', array('cat_id' => $category_id, 'group_id' => $category_group));
						$url = str_replace(SELF,$this->EE->config->item('cp_url'),$url);
					}

					if ($this->EE->TMPL->tagdata != '')
					{ 
						return str_replace('{edit_this_url}', $url, $this->EE->TMPL->tagdata);  
					}
					else 
					{	
						$alt = "Edit this category";
						return $this->_get_link($url, $alt, $this->EE->TMPL->fetch_param('icon'));
					}
				}
			}
			return;
		}
	}	

  // ----------------------------------------
  //  Plugin Usage
  // ----------------------------------------

  // This function describes how the plugin is used.
  //  Make sure and use output buffering

  function usage()
  {
  ob_start(); 
  ?>

This plugin will put small pencil icons in the live site, but only for users who are logged in, have an administrative session, and have permission to edit that particular resource.

All parameters are required, except "icon".

The simplest usage is simply this tag without any enclosed text. All of these will return an image and link as follows:
<a href="{edit_this_url}" title="Edit this" target="_blank"><img src="/themes/third_party/edit_this/images/default_icon.gif" height="14" width="14" class="edit_this" style="padding-left: 4px;" alt="Edit this" align="top" border="0" /></a>

You can over-ride styles on the image using the .edit_this class.

Entries:
{exp:edit_this:entry entry_id="{entry_id}" channel_id="{channel_id}" author_id="{author_id}"} 

Templates:
{exp:edit_this:template template_id="42" template_group="2"} 
(There's no way to dynamically insert the template_id or template_group, sadly.)

Categories:
{exp:edit_this:category category_id="{category_id}" category_group="{category_group}"}

If you wish to use a different icon image, several different colors are provided (in the /themes/third_party/edit_this/images directory).  You can even put your own image there, as long as the filename ends in “_icon.gif”.  Call alternate icons like so:
{exp:edit_this:template id="42" template_group="2" icon="blue"}

Advanced Use:
=============

You can use the same tag, but enclose some text within it.  The plugin will do the permission check, but leave the presentation of the link to you. This allows you to change the display to whatever you'd like, and you could have entire instructions or alternate content displayed by this plugin.

For example, you can point to a front-end editing template (stand alone editing form) using this option.  Or you could make the link be straight text, not an image.

{exp:edit_this:entry entry_id="{entry_id}" channel_id="{channel_id}" author_id="{author_id}"} 
<a href="{edit_this_url}">I can edit this</a>
{/exp:edit_this:entry}

{exp:edit_this:template template_id="42" template_group="2"} 
<a href="{edit_this_url}">Edit this template</a>
{/exp:edit_this:template}

{exp:edit_this:category category_id="{category_id}" category_group="{category_group}"}
<a href="{edit_this_url}" target="_blank">Click to modify category</a>
{/exp:edit_this:category}

Also, please note, you may need to set a cookie domain, or turn on cookies in the control panel, for edit_this to work.

Permissions Required

* For entries, users must be authorized to edit entries in the specific weblog; they must have control panel access; and they must have access to the edit tab.
* For templates, users must be authorized to edit templates in the specific template group; they must have control panel access; and they must be authorized to administer templates.
* For categories, users must have control panel access; they must be authorized to access the admin tab; and they must be authorized to administer weblogs.
* For all types of content, users must have an authenticated administrative session.

	<?php
  $buffer = ob_get_contents();
	
  ob_end_clean(); 

  return $buffer;
  }
  // END

}

/* End of file edit_this.php */
